@@ -103,11 +103,15 @@ proc merge_into_list {lname new_value {omit_value {}}} {
 
 proc file_report {} {
     global TkGnats Tkeditpr flds
-    if {[real_file_report] == -1} {
-        # bell
-        headingMsg "Error filing report!"
+    set stat [real_file_report]
+    if {$stat == 1} {
         headingMsg "" 0
-        return -1
+        return
+    }
+    if {$stat == -1} {
+        #headingMsg "Error filing report!"
+        headingMsg "" 0
+        return
     }
     bind . <Destroy> ""
     unlock_pr $Tkeditpr(prid)
@@ -145,9 +149,9 @@ proc real_file_report_socket {rep} {
         return -1
     }
 
-    write_pr   $s
-
-    puts $s "."
+    write_pr $s
+#write_pr stdout
+    puts     $s "."
 
     set rep [get_socket_reply $s]
     if {![string match 2* [lindex $rep 0]]} {
@@ -159,7 +163,7 @@ proc real_file_report_socket {rep} {
 }
 
 proc real_file_report {} {
-    global TkGnats Tkeditpr flds frm errorCode
+    global TkGnats Tkeditpr flds frm errorCode whyText
     flush_singletext $Tkeditpr(singletextflds)
     flush_multitext
 
@@ -169,162 +173,187 @@ proc real_file_report {} {
     
     headingMsg "Checking fields..." 0
 
-    # - Check Category field...
-    if {[lsearch -exact $TkGnats(CategoryList) $frm(>Category)] < 0} {
-        Msg "You have specified an invalid Category!"
-        return -1
-    }
-    # - Check Submitter field...
-    if {[lsearch -exact $TkGnats(SubmitterList) $frm(>Submitter-Id)] < 0} {
-        Msg "You have specified an invalid Submitter-Id!"
-        return -1
-    }
-    # - Check Responsible field...
-    if {[lsearch -exact $TkGnats(ResponsibleList) $frm(>Responsible)] < 0} {
-        Msg "You have specified an invalid Responsible!"
-        return -1
-    }
-
-    # - Check that indexed text fields do not have a | char. (hoses tkquerypr)
-    foreach t {>Release >Synopsis} {
-        if {[string first "|" [textget $t]] != -1} {
-            Msg " '|' is an illegal character for the '$t' field!"
+    # Check mandatory fields for non-blank input
+    foreach f [concat $Tkeditpr(radioflds) $Tkeditpr(listboxflds) $Tkeditpr(singletextflds) $Tkeditpr(multitextflds)] {
+        if {[check_suppressed_field $f]} {
+            continue
+        }
+	if {![check_mandatory_field $f]} {
+	    continue
+	}
+        set  val [string trim $frm($f) " \n\t\f!:~-#_?"]
+        if {$val == ""} {
+            Msg "You must supply a value for '[get_field_alias $f]'."
             return -1
         }
     }
 
+    # Check listbox fields
+    foreach {field list} {Category Category Submitter-Id Submitter Responsible Responsible} {
+        if {$frm(>$field) != "" && [lsearch -exact $TkGnats(${list}List) $frm(>$field)] < 0} {
+            Msg "You have specified an invalid [get_field_alias $field]: $frm(>$field)"
+            return -1
+        }
+    }
+
+    # Check Date-Required field...
+    if {$TkGnats(ReleaseBased)} {
+        if {[check_date_invalid $frm(>Date-Required) "Invalid [get_field_alias Date-Required]:"]} {
+            return -1
+        }
+    }
+    
+    set requireAudit  0
+    set requireEmail  0
+    set requireReason 0
+    
+    set    Audit_text "Changed-When:\t[clock format [clock seconds]]\n"
+    append Audit_text "Changed-By:\t$TkGnats(LogName) \"$TkGnats(FullName)\" <[lindex [extract_email_address $TkGnats(EmailAddr)] 0]>\n"
+    set    Email_text $Audit_text
+
     #
     # now see if any fields changed that trigger notifiers or audit records
     #
     
     set mail_list ""
-    set all_changes ""
-    set datestr [clock format [clock seconds]]
 
     set responsible_addr     [get_responsible_addr $frm(>Responsible)]
     set old_responsible_addr [get_responsible_addr $flds(>Responsible)]
  
-    set t ">Responsible"
-    if {$frm($t) != $flds($t)} {
-
-        # Mail to the old and new responsible persons
-        merge_into_list mail_list $old_responsible_addr $TkGnats(EmailAddr)
-        merge_into_list mail_list $responsible_addr $TkGnats(EmailAddr)
-
-        set change_msg   "\nResponsible-Changed-From-To: $flds($t)->$frm($t)\n"
-        append change_msg  "Responsible-Changed-By:      $TkGnats(FullName) <[lindex [extract_email_address $TkGnats(EmailAddr)] 0]>\n"
-        append change_msg  "Responsible-Changed-When:    $datestr\n"
-        set whyText ""
-        catch {
-            set whyText [entryDialog "Enter reason for RESPONSIBLE change" no_cancel]
+    foreach t [concat $Tkeditpr(listboxflds) $Tkeditpr(radioflds)] {
+        if {$frm($t) != $flds($t)} {
+            foreach require {Audit Email Reason} {
+                if {[check_audit_trail_opts $require $t]} {
+                    set  require$require 1
+                    if {$require != "Reason"} {
+                        append ${require}_text "Changed-[get_field_alias $t]-From-To:\t$flds($t)->$frm($t)\n"
+                    }
+                }
+            }
         }
-        append change_msg  "Responsible-Changed-Why:     $whyText\n"
-
-        append all_changes $change_msg 
-    }
- 
-    set t ">State"
-    if {$frm($t) != $flds($t)} {
-
-        # Mail to the responsible person
-        merge_into_list mail_list $responsible_addr $TkGnats(EmailAddr)
-
-        set change_msg    "\nState-Changed-From-To: $flds($t)->$frm($t)\n"
-        append change_msg   "State-Changed-By:      $TkGnats(FullName) <[lindex [extract_email_address $TkGnats(EmailAddr)] 0]>\n"
-        append change_msg   "State-Changed-When:    $datestr\n"
-        set whyText ""
-        catch {
-            set whyText [entryDialog "Enter reason for STATE change" no_cancel]
-        }
-        append change_msg   "State-Changed-Why:     $whyText\n"
-
-        append all_changes $change_msg 
-    }
- 
-    set t ">Priority"
-    if {$frm($t) != $flds($t)} {
-
-        # Mail to the responsible person
-        merge_into_list mail_list $responsible_addr $TkGnats(EmailAddr)
-
-        set change_msg    "\nPriority-Changed-From-To: $flds($t)->$frm($t)\n"
-        append change_msg   "Priority-Changed-By:      $TkGnats(FullName) <[lindex [extract_email_address $TkGnats(EmailAddr)] 0]>\n"
-        append change_msg   "Priority-Changed-When:    $datestr\n"
-        set whyText ""
-        catch {
-            set whyText [entryDialog "Enter reason for PRIORITY change" no_cancel]
-        }
-        append change_msg   "Priority-Changed-Why:     $whyText\n"
-
-        append all_changes $change_msg 
-    }
- 
-    set t ">Severity"
-    if {$frm($t) != $flds($t)} {
-
-        # Mail to the responsible person
-        merge_into_list mail_list $responsible_addr $TkGnats(EmailAddr)
-
-        set change_msg    "\nSeverity-Changed-From-To: $flds($t)->$frm($t)\n"
-        append change_msg   "Severity-Changed-By:      $TkGnats(FullName) <[lindex [extract_email_address $TkGnats(EmailAddr)] 0]>\n"
-        append change_msg   "Severity-Changed-When:    $datestr\n"
-        set whyText ""
-        catch {
-            set whyText [entryDialog "Enter reason for SEVERITY change" no_cancel]
-        }
-        append change_msg   "Severity-Changed-Why:     $whyText\n"
-
-        append all_changes $change_msg 
     }
 
+    foreach f $Tkeditpr(singletextflds) {
+	# Check that singletext text fields do not have a | char. (not allowed in gnats index)
+        if {[string first "|" $frm($f)] >= 0} {
+            Msg " '|' is an illegal character for the '[get_field_alias $f]' field!"
+            return -1
+        }
+        if {$frm($f) != $flds($f)} {
+            foreach require {Audit Email Reason} {
+                if {[check_audit_trail_opts $require $f]} {
+                    set require$require 1
+                    if {$require != "Reason"} {
+                        set tmptag [get_field_alias $f]
+                        append ${require}_text "Changed-$tmptag-From:\t$flds($f)\n"
+                        append ${require}_text "Changed-$tmptag-To:  \t$frm($f)\n"
+                    }
+                }
+            }
+        }
+    }
+
+    foreach t $Tkeditpr(multitextflds) {
+        if {[string trim $frm($t)] != [string trim $flds($t)]} {
+            foreach require {Audit Email Reason} {
+                if {[check_audit_trail_opts $require $t]} {
+                    set  require$require 1
+                    if {$require != "Reason"} {
+                        append ${require}_text "Changed-[get_field_alias $t]\n"
+                    }
+                }
+            }
+        }
+    }
+
+    headingMsg "" 0
+
+    if {$requireReason} {
+        if {[catch {textEntryDialog "Enter reason for changes" "Return to Edit" $whyText 0 .} whyText]} {
+            return 1
+        }
+        append Audit_text  "Changed-Why:\t[string trim $whyText]\n"
+        append Email_text  "Changed-Why:\t[string trim $whyText]\n"
+    }
+
+    if {$requireAudit} {
+        set dash "------------------------------------------------------------------\n"
+        append frm(>Audit-Trail) "\n$dash$Audit_text$dash"
+    }
+    
+    headingMsg "Filing report..." 0
+
+    set done 0
+    while {!$done} {
+        set stat [real_file_report_$TkGnats(GNATS_ACCESS_METHOD) errs]
+        #puts "stat=$stat errs=$errs"
+        if {$stat == 0} {
+            set done 1
+        } {
+            # If the failure is opening gnatsd socket, stat= -2
+            if {$stat != -2} {
+                # The first message is returned by gnatsd directly;
+                # the second by the batch [n]pr-edit.
+                if {[string first "currently locked" $errs] >= 0 || [string first "lock file exists" $errs] >= 0} {
+                    set errs "The GNATS database is presently locked.\nPlease try again in a moment."
+                }
+            }
+            set msg "Error filing $frm(>Category)/$Tkeditpr(prid):\n\n$errs"
+            
+            bell
+            set rep [tk_dialog .tkquerypr_delete "Error Filing PR Changes" $msg \
+                    "warning" 0 "Try Again" "Return to Edit"]
+            if {$rep == "1"} {
+                return -1
+            }
+        }
+    }
+
+    headingMsg "Done" 0
+    
     #
-    # did any notifiable changes take place ??
+    # Did any notifiable changes take place ??
     #
-    if {"$all_changes" != ""} {
-
-        # Send to the original sender and the Reply-To list
-        merge_into_list mail_list $flds(From)    $TkGnats(EmailAddr)
-        merge_into_list mail_list $frm(Reply-To) $TkGnats(EmailAddr)
+    
+    if {$requireEmail} {
+        # Send to the responsible person, original sender and the X-GNATS-Notify list
+        merge_into_list mail_list $responsible_addr    $TkGnats(EmailAddr)
+        merge_into_list mail_list $flds(Reply-To)      $TkGnats(EmailAddr)
+        merge_into_list mail_list $frm(X-GNATS-Notify) $TkGnats(EmailAddr)
+        if {$frm(>Responsible) != $flds(>Responsible)} {
+            # Mail to the old responsible person too
+            merge_into_list mail_list $old_responsible_addr $TkGnats(EmailAddr)
+        }
         ##puts "maillist=$mail_list"
         
         if {"$mail_list" != ""} {
-            headingMsg "Sending mail..." 0
-            set addrs [join $mail_list ", "]
+            headingMsg "Changes saved. Sending mail..." 0
+            set addrs [add_email_domainname [join $mail_list ", "]]
             ##puts "maillist=$addrs"
             set mailtxt ""
             append mailtxt "From: $TkGnats(EmailAddr)\n"
             append mailtxt "Reply-To: $TkGnats(EmailAddr)\n"
             append mailtxt "To: $addrs\n"
             append mailtxt "Subject: Re: $frm(>Category)/$Tkeditpr(prid): Changed information\n\n"
-            append mailtxt "Synopsis: $frm(>Synopsis)\n\n"
-            append mailtxt "Priority: $frm(>Priority)\n"
-            append mailtxt "Severity: $frm(>Severity)\n"
-            append mailtxt \n$all_changes
+            #TTD: add database name here!
+            foreach f {Synopsis Priority Severity} {
+                if {[check_suppressed_field $f]} {
+                    continue
+                }
+                set alias [get_field_alias $f]
+                append mailtxt "[get_field_alias $f]: $frm(>$f)\n"
+            }
+            #append mailtxt "Synopsis: $frm(>Synopsis)\n\n"
+            #append mailtxt "Priority: $frm(>Priority)\n"
+            #append mailtxt "Severity: $frm(>Severity)\n"
+            append mailtxt \n$Email_text
 
             if {[TkGnats_sendmail $addrs $mailtxt] == "-1"} {
-                Msg "Error sending mail notification.\n\nPR changes will still be saved."
+                Msg "Error sending mail notification.\n\nPR changes were saved."
             }
         }
-        append frm(>Audit-Trail) "\n$all_changes"
-    }
-    headingMsg "Filing report..." 0
-
-    set stat [real_file_report_$TkGnats(GNATS_ACCESS_METHOD) errs]
-    #puts "stat=$stat errs=$errs"    
-    if {$stat} {
-        # If the failure is opening gnatsd socket, stat= -2
-        if {$stat != -2} {
-            if {[string first "currently locked" $errs] >= 0} {
-                # GNATS is locked! Try again later.
-            } elseif {[string first "lock file exists" $errs] >= 0} {
-                set errs "GNATS is presently locked.\nPlease try again in a moment."
-            }
-            Msg "Error filing $frm(>Category)/$Tkeditpr(prid):\n" "$errs"
-        }
-        return -1
-    } {
-        headingMsg "Done" 0
     }
+    set whyText ""
     return 0
 }
 
@@ -354,12 +383,11 @@ proc TTDescape_dots_gnatsd {txt} {
 proc write_multitextfld {fout flds tag} {
     global TkGnats
     upvar 1 $flds f
-    set txt [string trimleft "$tag: \n[string trim $f($tag) "\n"]" "\n"]
+    #set txt [string trimleft "$tag: \n[string trim $f($tag) "\n"]" "\n"]
+    set txt [string trim "$tag: \n[string trim $f($tag) "\n"]" "\n"]
     if {$TkGnats(GNATS_ACCESS_METHOD) == "batch"} {
         puts $fout $txt
     } {
-        # TTD change this to escape_dots if gnatsd is ever fixed
-        # (it doesn't remove the extra dot at its end)
         puts $fout [escape_dots $txt]
     }
 }
@@ -374,19 +402,26 @@ proc write_pr {fout} {
     #
     # for each parsed field from the PR form...
     #
+#puts "tags:$Tkeditpr(parsed_flds)"
     foreach tag $Tkeditpr(parsed_flds) {
         case $tag {_prefix_} {
             #
             #   The mail header, stored under the _prefix_ tag, is written out
-            #   unadulterated, except for the edited Reply-To field.
+            #   unadulterated, except for the edited X-GNATS-Notify field.
             #
+#puts "prefix before:$flds($tag)"
             set lines [split $flds($tag) \n]
-            set repin [lsearch -regexp $lines "^Reply-To: "]
-            set repto "Reply-To: $frm(Reply-To)"
-            set newlines [lreplace $lines $repin $repin $repto]
-            set frm($tag) [join $newlines \n]
-            puts $fout $frm($tag) nonewline
-        } {Reply-To} {
+            set idx   [lsearch -regexp $lines "^X-GNATS-Notify:"]
+            set repto "X-GNATS-Notify: [lindex [split $frm(X-GNATS-Notify) ,] 0]"
+            foreach addr [lrange [split $frm(X-GNATS-Notify) ,] 1 end] {
+                append repto ", [string trim $addr]"
+            }
+            set lines [lreplace $lines $idx [expr $idx + $flds(_prefix_len_X-GNATS-Notify) - 1] $repto\n]
+            #set frm($tag) "[string trim [join $lines \n]]\n"
+            set frm($tag) [join $lines \n]
+#puts "prefix after:$flds($tag)"
+            puts -nonewline $fout $frm($tag)
+        } {X-GNATS-Notify} {
             #
             # Taken care of above with the mail header
             #
@@ -418,15 +453,14 @@ proc write_pr {fout} {
             #   tab and a trailing newline
             #
             case $tag $Tkeditpr(singletextflds) {
-                if {$tag != "Reply-To"} {
-                    puts $fout "$tag:\t[string trim [textget $tag] "\t\n "]"
-                }
+                #puts $fout "$tag:\t[string trim [textget $tag] "\t\n "]"
+                puts $fout "$tag:\t[string trim $frm($tag) "\t\n "]"
             } [concat >Category >Responsible >Submitter-Id $Tkeditpr(radioflds)] {
                 puts $fout "$tag:\t$frm($tag)"
             } $Tkeditpr(multitextflds) {
                 write_multitextfld $fout frm $tag
             } default {
-                puts $fout "$tag:$data" nonewline
+                puts -nonewline $fout "$tag:$data"
             }
         }
     }
@@ -470,22 +504,22 @@ proc get_tkeditpr_listbox_height {} {
     set Tkeditpr(listbox_height) $h
 }
     
-proc edit_category_listbox {p {pat *} {valwid 0}} {
+proc edit_category_listbox {p {pat *}} {
     global TkGnats Tkeditpr flds frm
-    if {$valwid == 0} {
-        set valwid [string length [ftrim $flds(>Category)]]
-    }
-
     set wid [expr 2 + [get_max_strlen $TkGnats(CategoryList)]]
-    set Tkeditpr(listbox_width) $wid
-    
+    set alias [get_field_alias Category]
     frame  $p.cat -relief flat
     pack   $p.cat -side top -anchor w
-    button $p.cat.lab -text "Category: " -width 14 -anchor w -command "helpMsg Category" \
+    button $p.cat.lab -text "${alias}: " -width 14 -anchor w -command "helpMsg $alias" \
             -relief flat -padx 0 -pady 0 -borderwidth 0 -highlightthickness 1
     frame  $p.cat.msg -relief flat
     label  $p.cat.msg.val -text "[ftrim $flds(>Category)]" -relief groove -anchor w \
             -width $wid -background $TkGnats(ReadOnlyBackground)
+    pack $p.cat.msg.val  -side top   -fill both -expand true -anchor center
+    pack $p.cat.lab $p.cat.msg -side left -anchor n
+    if {[check_suppressed_field Category] == 2} {
+        return ""
+    }
     set ew [entry $p.cat.msg.ent -width $wid -insertwidth 1 -insertofftime 400 \
             -relief sunken -borderwidth 2 -background $TkGnats(EditFieldBackground) -highlightthickness 2 \
             -textvariable frm(>Category)]
@@ -496,31 +530,31 @@ proc edit_category_listbox {p {pat *} {valwid 0}} {
     scrollbar $p.cat.msg.sb -command "$p.cat.msg.list yview" -borderwidth 2 -relief sunken
     listbox $p.cat.msg.list -yscroll "$p.cat.msg.sb set" -setgrid 1 -relief sunken -borderwidth 2 \
             -width $wid -height $Tkeditpr(listbox_height) -exportselection false
-    pack $p.cat.msg.val  -side top   -fill both -expand true -anchor center
     pack $p.cat.msg.ent  -side top   -fill both -expand true -anchor w
     pack $p.cat.msg.list -side left  -fill both -expand true
     pack $p.cat.msg.sb   -side right -fill y
     eval $p.cat.msg.list insert end $TkGnats(CategoryList)
     trace variable frm(>Category) w set_edit_category_ew
-    pack $p.cat.lab $p.cat.msg -side left -anchor n
     bind   $p.cat.msg.list <B1-ButtonRelease> "set_edit_category $p.cat.msg %W %y"
     return $p.cat.msg.list
 }
 
-proc edit_responsible_listbox {p {pat *} {valwid 0}} {
+proc edit_responsible_listbox {p {pat *}} {
     global TkGnats Tkeditpr flds frm
-    if {$valwid == 0} {
-        set valwid [string length [ftrim $flds(>Responsible)]]
-    }
-
     set wid [expr 2 + [get_max_strlen $TkGnats(ResponsibleList)]]
+    set alias [get_field_alias Responsible]
     frame  $p.res -relief flat
     pack   $p.res -side top -anchor w
-    button $p.res.lab -text "Responsible: " -width 14 -anchor w -command "helpMsg Responsible" \
+    button $p.res.lab -text "${alias}: " -width 14 -anchor w -command "helpMsg $alias" \
             -relief flat -padx 0 -pady 0 -borderwidth 0 -highlightthickness 1
     frame  $p.res.msg -relief flat
     label  $p.res.msg.val -text "[ftrim $flds(>Responsible)]" -relief groove -anchor w \
             -width $wid -background $TkGnats(ReadOnlyBackground)
+    pack $p.res.msg.val  -side top   -fill both -expand true -anchor center
+    pack $p.res.lab $p.res.msg -side left -anchor n
+    if {[check_suppressed_field Responsible] == 2} {
+        return ""
+    }
     set ew [entry $p.res.msg.ent -width $wid -insertwidth 1 -insertofftime 400 \
             -relief sunken -borderwidth 2 -background $TkGnats(EditFieldBackground) -highlightthickness 2 \
             -textvariable frm(>Responsible)]
@@ -535,30 +569,31 @@ proc edit_responsible_listbox {p {pat *} {valwid 0}} {
     }
     listbox $p.res.msg.list -yscroll "$p.res.msg.sb set" -setgrid 1 -relief sunken \
             -borderwidth 2 -width $wid -height $Tkeditpr(listbox_height) -exportselection false
-    pack $p.res.msg.val  -side top   -fill both -expand true -anchor center
     pack $p.res.msg.ent  -side top   -fill both -expand true -anchor w
     pack $p.res.msg.list -side left  -fill both -expand true
     pack $p.res.msg.sb   -side right -fill y
     eval $p.res.msg.list insert end $TkGnats(ResponsibleList)
     trace variable frm(>Responsible) w set_edit_responsible_ew
-    pack $p.res.lab $p.res.msg -side left -anchor n
     bind   $p.res.msg.list <B1-ButtonRelease> "set_edit_responsible $p.res.msg %W %y"
     return $p.res.msg.list
 }
 
-proc edit_submitter_listbox {p {pat *} {valwid 0}} {
+proc edit_submitter-id_listbox {p {pat *}} {
     global TkGnats Tkeditpr flds frm
-    if {$valwid == 0} {
-        set valwid [string length [ftrim $flds(>Submitter-Id)]]
-    }
     set wid [expr 2 + [get_max_strlen $TkGnats(SubmitterList)]]
+    set alias [get_field_alias Submitter-Id]
     frame  $p.sub -relief flat
     pack   $p.sub -side top -anchor w
-    button $p.sub.lab -text "Submitter-Id: " -width 14 -anchor w -command "helpMsg Submitter-Id" \
+    button $p.sub.lab -text "${alias}: " -width 14 -anchor w -command "helpMsg $alias" \
             -relief flat -padx 0 -pady 0 -borderwidth 0 -highlightthickness 1
     frame  $p.sub.msg -relief flat
     label  $p.sub.msg.val -text "[ftrim $flds(>Submitter-Id)]" -relief groove -anchor w \
             -width $wid -background $TkGnats(ReadOnlyBackground)
+    pack $p.sub.msg.val  -side top   -fill both -expand true -anchor center
+    pack $p.sub.lab $p.sub.msg -side left -anchor n
+    if {[check_suppressed_field Submitter-Id] == 2} {
+        return ""
+    }
     set ew [entry $p.sub.msg.ent -width $wid -insertwidth 1 -insertofftime 400 \
             -relief sunken -borderwidth 2 -background $TkGnats(EditFieldBackground) \
             -highlightthickness 2 -textvariable frm(>Submitter-Id)]
@@ -573,14 +608,12 @@ proc edit_submitter_listbox {p {pat *} {valwid 0}} {
     }
     listbox $p.sub.msg.list -yscroll "$p.sub.msg.sb set" -setgrid 1 -relief sunken \
             -borderwidth 2 -width $wid -height $Tkeditpr(listbox_height) -exportselection false
-    pack $p.sub.msg.val  -side top   -fill both -expand true -anchor center
     pack $p.sub.msg.ent  -side top   -fill both -expand true -anchor w
     pack $p.sub.msg.list -side left  -fill both -expand true
     pack $p.sub.msg.sb   -side right -fill y
     eval $p.sub.msg.list insert end $TkGnats(SubmitterList)
-    trace variable frm(>Submitter-Id) w set_edit_submitter_ew
-    pack $p.sub.lab $p.sub.msg -side left -anchor n
-    bind   $p.sub.msg.list <B1-ButtonRelease> "set_edit_submitter $p.sub.msg %W %y"
+    trace variable frm(>Submitter-Id) w set_edit_submitter-id_ew
+    bind   $p.sub.msg.list <B1-ButtonRelease> "set_edit_submitter-id $p.sub.msg %W %y"
     return $p.sub.msg.list
 }
 
@@ -591,7 +624,7 @@ proc set_edit_category_ew {a b c} {
             $TkGnats(CategoryList)
 }
 
-proc set_edit_submitter_ew {a b c} {
+proc set_edit_submitter-id_ew {a b c} {
     upvar #0 $a f
     global TkGnats
     quickfill_entry_from_listbox f($b) .eboxs.slb.sub.msg.ent .eboxs.slb.sub.msg.list \
@@ -623,36 +656,36 @@ proc set_edit_responsible {msg w y} {
     trace variable frm(>Responsible) w set_edit_responsible_ew
 }
 
-proc set_edit_submitter {msg w y} {
+proc set_edit_submitter-id {msg w y} {
     global frm
-    trace vdelete  frm(>Submitter-Id) w set_edit_submitter_ew
+    trace vdelete  frm(>Submitter-Id) w set_edit_submitter-id_ew
     $msg.ent delete 0 end
     set idx [$w nearest $y]
     set frm(>Submitter-Id) [$w get $idx]
-    trace variable frm(>Submitter-Id) w set_edit_submitter_ew
+    trace variable frm(>Submitter-Id) w set_edit_submitter-id_ew
 }
 
 proc fillfrm {} {
-    global Tkeditpr flds frm
+    global TkGnats Tkeditpr flds frm
 
     ### re-set editable PR values to values currently in the PR
 
     # clear listbox entry widgets
     trace vdelete  frm(>Category)     w set_edit_category_ew
-    trace vdelete  frm(>Submitter-Id) w set_edit_submitter_ew
+    trace vdelete  frm(>Submitter-Id) w set_edit_submitter-id_ew
     trace vdelete  frm(>Responsible)  w set_edit_responsible_ew
-    .eboxs.clb.cat.msg.ent delete 0 end
-    .eboxs.slb.sub.msg.ent delete 0 end
-    .eboxs.rlb.res.msg.ent delete 0 end
+    catch {.eboxs.clb.cat.msg.ent delete 0 end}
+    catch {.eboxs.slb.sub.msg.ent delete 0 end}
+    catch {.eboxs.rlb.res.msg.ent delete 0 end}
     trace variable frm(>Category)     w set_edit_category_ew
-    trace variable frm(>Submitter-Id) w set_edit_submitter_ew
+    trace variable frm(>Submitter-Id) w set_edit_submitter-id_ew
     trace variable frm(>Responsible)  w set_edit_responsible_ew
 
     # listboxes and radio (enumerated) fields
-    foreach tag [concat >Category >Responsible >Submitter-Id $Tkeditpr(radioflds)] {
+    foreach tag [concat $Tkeditpr(listboxflds) $Tkeditpr(radioflds)] {
         set  flds($tag) [string trim $flds($tag) "\t\n "]
         #unset frm($tag)
-        set   frm($tag) $flds($tag)
+        catch {set frm($tag) $flds($tag)}
     }
 
     # now the 1 line textual flds
@@ -664,14 +697,14 @@ proc fillfrm {} {
     # now the multi line textual flds
     foreach tag $Tkeditpr(multitextflds) {
         if {[info exists flds($tag)]} {
-            set flds($tag) [string trim $flds($tag) "\n"]
-            set frm($tag) $flds($tag)
+            set flds($tag) [string trim $flds($tag) "\t\n "]
+            set frm($tag) $flds($tag)\n
         } {
             set frm($tag)  "\n"
             set flds($tag) "\n"
         }
     }
-    switch_txt ">Description" $Tkeditpr(multitextflds)
+    switch_txt $TkGnats(first_multitext) $Tkeditpr(multitextflds)
 }
 
 proc reset_report {} {
@@ -685,6 +718,14 @@ proc reset_report {} {
     switch_txt $current $Tkeditpr(multitextflds)
 }
 
+proc edit_email_originator {} {
+    global Tkeditpr flds frm
+    flush_singletext $Tkeditpr(singletextflds)
+    email_originator [ftrim $frm(X-GNATS-Notify)] \
+            [ftrim $frm(>Responsible)] [ftrim $flds(Reply-To)] \
+            [ftrim $frm(>Category)]/$Tkeditpr(prid) [ftrim $frm(>Synopsis)]
+}
+
 proc edit_cleanup {} {
     global Tkeditpr
     bind . <Destroy> ""
@@ -693,7 +734,7 @@ proc edit_cleanup {} {
 }
 
 proc edit_window {} {
-    global TkGnats Tkeditpr flds env current_multi_text
+    global TkGnats Tkeditpr flds frm env current_multi_text
 
     set prid $Tkeditpr(prid)
     
@@ -715,17 +756,21 @@ proc edit_window {} {
         >Class
     }
     
+    set Tkeditpr(listboxflds) {
+        >Category
+        >Submitter-Id
+        >Responsible
+    }
+    
     set Tkeditpr(singletextflds) {
         >Originator
-        Reply-To
+        X-GNATS-Notify
         >Release
         >Synopsis
     }
 
     if {$TkGnats(ReleaseBased)} {
-        lappend Tkeditpr(singletextflds) >Keywords
-        lappend Tkeditpr(singletextflds) >$TkGnats(Quarter)
-        lappend Tkeditpr(singletextflds) >Date-Required
+        lappend Tkeditpr(singletextflds) >Keywords >$TkGnats(Quarter) >Date-Required
     }
     
     set Tkeditpr(multitextflds) {
@@ -733,9 +778,9 @@ proc edit_window {} {
         >How-To-Repeat
         >Environment
         >Organization
+        >Fix
         >Audit-Trail
         >Unformatted
-        >Fix
         >Release-Note
     }
     
@@ -755,15 +800,19 @@ proc edit_window {} {
     menu       .mframe.file.m
     .mframe.file.m add command -label "Save Changes"              -command file_report
     .mframe.file.m add command -label "Reset to Starting Values"  -command reset_report
-    .mframe.file.m add command -label "Send Email..."  -command \
-            "email_originator [list [ftrim $flds(Reply-To)]] \
-            [list [ftrim $flds(>Responsible)]] [list [ftrim $flds(From)]] \
-            [ftrim $flds(>Category)]/$Tkeditpr(prid) [list [ftrim $flds(>Synopsis)]]"
+#    .mframe.file.m add command -label "Send Email..."  -command \
+#            "email_originator [list [ftrim $flds(X-GNATS-Notify)]] \
+#            [list [ftrim $flds(>Responsible)]] [list [ftrim $flds(Reply-To)]] \
+#            [ftrim $flds(>Category)]/$Tkeditpr(prid) [list [ftrim $flds(>Synopsis)]]"
+    .mframe.file.m add command -label "Send Email..."  -command edit_email_originator
     .mframe.file.m add separator
     .mframe.file.m add command -label "Cancel"                    -command cancel_report
     
     menubutton .mframe.edit -text "Edit" -menu .mframe.edit.m -underline 0
     menu       .mframe.edit.m
+    .mframe.edit.m     configure -disabledforeground [.mframe.edit.m cget -foreground]
+    .mframe.edit.m add command -label "Use right mouse button for Cut/Copy/Paste" -state disabled
+    .mframe.edit.m add separator
     .mframe.edit.m add command -label "Fonts..." -command "edit_fonts"
     
     pack .mframe.file .mframe.edit -side left
@@ -773,6 +822,9 @@ proc edit_window {} {
     .mframe.help.m add command -label "Overview" \
             -command "helpMsg Edit_Overview"
     .mframe.help.m add separator
+    .mframe.help.m add command -label "Cut, Copy, Paste Operations" \
+            -command "helpMsg Cut_Copy_Paste"
+    .mframe.help.m add separator
     .mframe.help.m add command -label "Field Definitions" \
             -command "helpMsg Field_Definitions"
     .mframe.help.m add separator
@@ -788,6 +840,8 @@ proc edit_window {} {
     .mframe.help.m add command -label "View Configuration Variables" \
         -command "helpMsg TkGnats_Variables"
     .mframe.help.m add separator
+    .mframe.help.m add command -label "Changes" \
+            -command "helpMsg Changes"
     .mframe.help.m add command -label "About" \
             -command "helpMsg TkGnats_About"
     pack       .mframe.help -side right
@@ -797,10 +851,11 @@ proc edit_window {} {
     button .action.send   -borderwidth 1 -text "Save Changes"              -command file_report
     #button .action.cancel -borderwidth 1 -text "Cancel"                    -command cancel_report
     button .action.reset  -borderwidth 1 -text "Reset to Starting Values"  -command reset_report
-    button .action.email  -borderwidth 1 -text "Send Email..."             -command \
-            "email_originator [list [ftrim $flds(Reply-To)]] [list [ftrim $flds(>Responsible)]] \
-            [list [ftrim $flds(From)]] [ftrim $flds(>Category)]/$Tkeditpr(prid) \
-            [list [ftrim $flds(>Synopsis)]]"
+    button .action.email  -borderwidth 1 -text "Send Email..."             -command edit_email_originator
+#    button .action.email  -borderwidth 1 -text "Send Email..."             -command \
+#            "email_originator [list [ftrim $flds(X-GNATS-Notify)]] \
+#            [list [ftrim $flds(>Responsible)]] [list [ftrim $flds(Reply-To)]] \
+#            [ftrim $flds(>Category)]/$Tkeditpr(prid) [list [ftrim $flds(>Synopsis)]]"
     
     #pack .action.send .action.reset .action.email .action.cancel -side left -padx 0
     pack .action.send .action.reset .action.email -side left -padx 0
@@ -810,7 +865,7 @@ proc edit_window {} {
 
     # Get the maximum width of the value fields for the bagged_radiobar
     set Tkeditpr(value_width) 0
-    foreach tag [concat >Category >Responsible $Tkeditpr(radioflds)] {
+    foreach tag $Tkeditpr(radioflds) {
         set f [ftrim $flds($tag)]
         if {[string length $f] > $Tkeditpr(value_width)} {
             set Tkeditpr(value_width) [string length $f]
@@ -818,8 +873,6 @@ proc edit_window {} {
     }
     
     # Get the maximum width of the value fields for the readonly_singletext fields
-    ###set Tkeditpr(singletextvalue_width) \
-            ###        [string length "[ftrim $flds(>Originator)] (email: $flds(Reply-To))"]
     set Tkeditpr(singletextvalue_width) 0
     set flist {>Last-Modified >Arrival-Date >Closed-Date}
     foreach tag $flist {
@@ -829,54 +882,44 @@ proc edit_window {} {
         }
     }
     
-    ###readonly_singletext Originator   "[ftrim $flds(>Originator)] (email: $flds(Reply-To))"  14 \
-            ###        $Tkeditpr(singletextvalue_width)
-    readonly_singletext Arrival-Date  [ftrim $flds(>Arrival-Date)]  14 \
-            $Tkeditpr(singletextvalue_width)
-    readonly_singletext Last-Modified [ftrim $flds(>Last-Modified)] 14 \
-            $Tkeditpr(singletextvalue_width)
-    readonly_singletext Closed-Date   [ftrim $flds(>Closed-Date)]   14 \
-            $Tkeditpr(singletextvalue_width)
+    foreach efield [list Arrival-Date Last-Modified Closed-Date] {
+        if {[check_suppressed_field $efield] != 1} {
+            # We don't want to support editing these for now.
+            readonly_singletext $efield [ftrim $flds(>$efield)] 14 $Tkeditpr(singletextvalue_width)
+        }
+    }
 
     frame .eflds
     radiobar_frame .eflds .eflds.lb
-    bagged_radiobar .eflds.lb class Class \
-            $TkGnats(ClassesList) None frm $Tkeditpr(value_width)
-#####            {sw-bug doc-bug change-request support mistaken duplicate} None frm $Tkeditpr(value_width)
-    bagged_radiobar .eflds.lb state State \
-            $TkGnats(StatesList)  None frm $Tkeditpr(value_width)
-#####            {open analyzed feedback closed suspended} None frm $Tkeditpr(value_width)
-    bagged_radiobar .eflds.lb priority Priority \
-            {low medium high} None frm $Tkeditpr(value_width)
-    bagged_radiobar .eflds.lb severity Severity \
-            {non-critical serious critical} None frm $Tkeditpr(value_width)
-    bagged_radiobar .eflds.lb confidential Confidential \
-            {no yes} None frm $Tkeditpr(value_width)
+    foreach {efield elist} [list Class $TkGnats(ClassesList) State $TkGnats(StatesList) Priority {low medium high} Severity {non-critical serious critical} Confidential {no yes}] {
+        if {[check_suppressed_field $efield] != 1} {
+            bagged_radiobar .eflds.lb [string tolower $efield] $efield \
+                    $elist None frm $Tkeditpr(value_width)
+        }
+    }
     pack .eflds.lb -side left -pady 0
     pack .eflds    -side top  -pady 0 -fill x -anchor w
     
     frame .eboxs
-    frame .eboxs.clb -relief groove -borderwidth 2
-    edit_category_listbox .eboxs.clb * $Tkeditpr(value_width)
-    frame .eboxs.slb -relief groove -borderwidth 2
-    edit_submitter_listbox   .eboxs.slb * $Tkeditpr(value_width)
-    frame .eboxs.rlb -relief groove -borderwidth 2
-    edit_responsible_listbox .eboxs.rlb * $Tkeditpr(value_width)
+    set   nboxes 1
+    array set xpad {1 0 2 20 3 0}
+    foreach {efield eframe} {Category clb Submitter-Id slb Responsible rlb} {
+        if {[check_suppressed_field $efield] != 1} {
+            frame .eboxs.$eframe -relief groove -borderwidth 2
+            edit_[string tolower $efield]_listbox .eboxs.$eframe *
+            pack  .eboxs.$eframe -side left -anchor nw -pady 2 -padx $xpad($nboxes) -fill y
+            incr nboxes
+        }
+    }
+    if {$nboxes > 1} {
+        pack .eboxs -side top  -anchor w -pady 2 -padx  0 -fill x
+    }
     
-    pack .eboxs.clb -side left -anchor w -pady 2 -padx  0
-    pack .eboxs.slb -side left -anchor w -pady 2 -padx 20
-    pack .eboxs.rlb -side left -anchor w -pady 2 -padx  0
-    pack .eboxs     -side top  -anchor w -pady 2 -padx  0 -fill x
-    
-    lappend Tkeditpr(tlist) [singletext . >Originator  80 "" 14]
-    lappend Tkeditpr(tlist) [singletext . Reply-To     80 "" 14]
-    lappend Tkeditpr(tlist) [singletext . >Release     80 "" 14]
-    lappend Tkeditpr(tlist) [singletext . >Synopsis    80 "" 14]
-
-    if {$TkGnats(ReleaseBased)} {
-        lappend Tkeditpr(tlist) [singletext . >Keywords          80 "" 14]
-        lappend Tkeditpr(tlist) [singletext . >$TkGnats(Quarter) 80 "" 14]
-        lappend Tkeditpr(tlist) [singletext . >Date-Required     80 "" 14]
+    foreach f $Tkeditpr(singletextflds) {
+        if {[check_suppressed_field $f] == 1} {
+            continue
+        }
+	lappend Tkeditpr(tlist) [singletext . $f 80 "" 14]
     }
     
     lappend Tkeditpr(tlist) [make_txt_mb $Tkeditpr(multitextflds)]
@@ -898,7 +941,8 @@ proc edit_window {} {
 
 ##################################################################
 
-set Tkeditpr(prid)           ""
+set whyText      ""
+set Tkeditpr(prid) ""
 
 tkeditpr_process_args
 
