@@ -148,7 +148,7 @@ array set list_flds_defaults  [list \
 ]
 
 array set gnatsd_commands [list \
-        Submitter    SUBM \
+        Submitter-Id SUBM \
         Originator   ORIG \
         Responsible  RESP \
         Category     CATG \
@@ -303,9 +303,10 @@ proc category_listbox {parent} {
 
     if {![winfo exists $parent.tit]} {
         set wid [expr 2 + [get_max_strlen $TkGnats(CategoryList)]]
-        
-        button  $parent.tit -anchor center -text "Category:" \
-                -command "helpMsg Category" -relief flat -padx 0 -pady 0 -borderwidth 0
+
+        set alias [get_field_alias Category]
+        button  $parent.tit -anchor center -text "${alias}:" \
+                -command "helpMsg $alias" -relief flat -padx 0 -pady 0 -borderwidth 0
         pack    $parent.tit -side top -fill x
         
         frame   $parent.cat
@@ -356,14 +357,15 @@ proc category_listbox {parent} {
     eval $parent.cat.l.list insert end $TkGnats(CategoryList)
 }
 
-proc submitter_listbox {parent} {
-    global TkGnats Query Submitter Submitter_regexp
+proc submitter-id_listbox {parent} {
+    global TkGnats Query Submitter-Id Submitter-Id_regexp
     
     if {![winfo exists $parent.tit]} {
         set wid [expr 2 + [get_max_strlen $TkGnats(SubmitterList)]]
         
-        button  $parent.tit -anchor center -text "Submitter-Id:" \
-                -command "helpMsg Submitter-Id" -relief flat -padx 0 -pady 0 -borderwidth 0
+        set alias [get_field_alias Submitter-Id]
+        button  $parent.tit -anchor center -text "${alias}:" \
+                -command "helpMsg $alias" -relief flat -padx 0 -pady 0 -borderwidth 0
         pack    $parent.tit -side top -fill x
         
         frame   $parent.sub 
@@ -391,25 +393,25 @@ proc submitter_listbox {parent} {
         }
         
         bind $parent.sub.l.list <B1-ButtonRelease> \
-                "submitter_add_cmd %W %y $parent.sub.r.list"
+                "submitter-id_add_cmd %W %y $parent.sub.r.list"
         
         bind $parent.sub.r.list <B1-ButtonRelease> \
-                "submitter_delete_cmd %W %y $parent.sub.l.list"
+                "submitter-id_delete_cmd %W %y $parent.sub.l.list"
         
         frame $parent.reg
         pack  $parent.reg -side top -fill x
         
         lappend Query(tlist) [singletext $parent.reg "RegExp" 10 "" 7]
-        set Submitter_regexp $parent.reg
+        set Submitter-Id_regexp $parent.reg
     } {
         # Just clear out the existing widgets
         $parent.sub.l.list delete 0 end 
         $parent.sub.r.list delete 0 end 
-        unset Submitter
+        unset Submitter-Id
     }
     
-    # Just a place holder so that Submitter is defined as an array
-    set Submitter(All) ""
+    # Just a place holder so that Submitter-Id is defined as an array
+    set Submitter-Id(All) ""
     
     eval $parent.sub.l.list insert end $TkGnats(SubmitterList)
 }
@@ -420,8 +422,9 @@ proc responsible_listbox {parent} {
     if {![winfo exists $parent.tit]} {
         set wid [expr 2 + [get_max_strlen $TkGnats(ResponsibleList)]]
         
-        button  $parent.tit -anchor center -text "Responsible:" \
-                -command "helpMsg Responsible" -relief flat -padx 0 -pady 0 -borderwidth 0
+        set alias [get_field_alias Responsible]
+        button  $parent.tit -anchor center -text "${alias}:" \
+                -command "helpMsg $alias" -relief flat -padx 0 -pady 0 -borderwidth 0
         pack    $parent.tit -side top -fill x
         
         frame   $parent.res 
@@ -497,17 +500,17 @@ proc category_delete_cmd {srcw y destw} {
     sort_listbox $destw
 }
 
-proc submitter_add_cmd {srcw y destw} {
-    global Submitter
+proc submitter-id_add_cmd {srcw y destw} {
+    global Submitter-Id
     set val [list_item_switch_cmd $srcw $y $destw]
-    set Submitter($val) $val
+    set Submitter-Id($val) $val
 }
 
-proc submitter_delete_cmd {srcw y destw} {
-    global Submitter
+proc submitter-id_delete_cmd {srcw y destw} {
+    global Submitter-Id
     set val [list_item_switch_cmd $srcw $y $destw]
     if {"$val" != ""} {
-	unset Submitter($val)
+	unset Submitter-Id($val)
     }
     sort_listbox $destw
 }
@@ -535,7 +538,8 @@ proc set_query_view_heading {} {
     for {set i 2} {$i < [llength $list_flds_list]} {incr i 2} {
         set x [lindex $list_flds_list $i]
         if {$list_flds_selected($x) != ""} {
-            append  list_flds_heading [format $list_flds_formats($x) $list_flds_headings($x)]
+            #append  list_flds_heading [format $list_flds_formats($x) $list_flds_headings($x)]
+            append  list_flds_heading [format $list_flds_formats($x) [get_field_alias $list_flds_headings($x)]]
             lappend list_flds_widths  [string trim $list_flds_formats($x) "%- s"]
         }
     }
@@ -556,29 +560,38 @@ proc update_query_view_fields {} {
 proc set_query_view_fields {} {
     global Query list_flds_selected list_flds_heading
     set_query_view_heading
-    .qlb.query.label configure -text $list_flds_heading
+    set Query(lbheading) $list_flds_heading
 }
 
 proc query_listbox {p} {
-    global TkGnats Category list_flds_heading tcl_platform
+    global TkGnats Query Category list_flds_heading tcl_platform
     set lboxwidth  100
+    set Query(maxlen) $lboxwidth
+    #set Query(lbheading) $list_flds_heading
     frame $p.query 
     pack  $p.query -side top -fill both -expand true
     set_query_view_heading
+    button  $p.temp
+    set bg [$p.temp cget -background]
+    destroy $p.temp
     frame   $p.query.top
-    label   $p.query.label -font $TkGnats(textfont) -anchor w -text $list_flds_heading \
-            -borderwidth 2 -highlightthickness 0
+    entry   $p.query.entry -font $TkGnats(textfont) -textvar Query(lbheading) -background $bg \
+            -borderwidth 2 -highlightthickness 0 -state disabled -relief flat -cursor left_ptr
+    bind $p.query.entry <1> "break"
+    bind $p.query.entry <B1-Motion> "break"
+    bind $p.query.entry <2> "break"
+    bind $p.query.entry <B2-Motion> "break"
     listbox $p.query.list  -font $TkGnats(textfont) -relief sunken -setgrid 1 \
-            -xscroll "$p.query.sbx set" -yscroll "$p.query.sby set" \
+            -xscroll "query_listbox_xscroll $p.query.sbx $p.query.list" \
+            -yscroll "$p.query.sby set"  -takefocus 1 \
             -width ${lboxwidth} -height 12 -exportselection false \
             -borderwidth 2 -highlightthickness 2
     scrollbar $p.query.sby -command "$p.query.list yview" \
-            -borderwidth 2 -highlightthickness 2 -orient vertical
-    
+            -borderwidth 2 -highlightthickness 2 -orient vertical -takefocus 0
     # Create padding based on the y scrollbar width and border
     frame $p.query.bottom
     scrollbar $p.query.sbx -command "$p.query.list xview" \
-            -borderwidth 2 -highlightthickness 2 -orient horizontal
+            -borderwidth 2 -highlightthickness 2 -orient horizontal -takefocus 0
 
     if {$tcl_platform(platform) == "unix"} {
         set pad2 [expr 2 * ([$p.query.sby cget -bd] + [$p.query.sby cget -highlightthickness])]
@@ -592,7 +605,7 @@ proc query_listbox {p} {
     frame $p.query.padb -width $pad -height $pad
 
     pack  $p.query.padt   -in $p.query.top -side left
-    pack  $p.query.label  -in $p.query.top -side left -fill both -anchor w -expand true \
+    pack  $p.query.entry  -in $p.query.top -side left -fill x -anchor w -expand true \
 	    -padx [$p.query.list cget -highlightthickness]
     pack  $p.query.top    -side top    -fill x 
     
@@ -603,25 +616,74 @@ proc query_listbox {p} {
     pack  $p.query.sby    -side left   -fill y 
     pack  $p.query.list   -side right  -fill both -expand true
 
-    bind  $p.query.list <Double-Button-1>  "selection_View_Formatted_cmd %W"
+    # I think this is a bug in the listbox widget. Prior and Next are missing the selection set.
+    bind Listbox <Prior> {
+        %W yview scroll -1 pages
+        %W activate @0,0
+        %W selection clear 0 end
+        %W selection set active
+    }
+    bind Listbox <Next> {
+        %W yview scroll 1 pages
+        %W activate @0,0
+        %W selection clear 0 end
+        %W selection set active
+    }
+    bind Listbox <ButtonRelease-1> {
+        tkCancelRepeat
+        %W activate @%x,%y
+        %W selection clear 0 end
+        %W selection set active
+    }
+    
+    if {[info exists TkGnats(QueryDefaultAction)]  && \
+            $TkGnats(QueryDefaultAction) == "edit" && $TkGnats(edit_authorized)} {
+        bind  $p.query.list <Double-Button-1>  "selection_Edit_cmd $p.query.list"
+        bind  $p.query.list <Return>           "selection_Edit_cmd $p.query.list"
+    } {
+        set  TkGnats(QueryDefaultAction) view
+        bind  $p.query.list <Double-Button-1>  "selection_View_Formatted_cmd %W"
+        bind  $p.query.list <Return>           "selection_View_Formatted_cmd %W"
+    }
     bind  $p.query.list <Control-l>        "%W xview 0"
     bind  $p.query.list <KeyRelease-Left>  "%W xview 0"
     bind  $p.query.list <KeyRelease-Right> "%W xview [expr $lboxwidth/2]"
     bind  $p.query.list <Control-r>        "%W xview [expr $lboxwidth/2]"
     bind  $p.query.list <3> {
-        tkListboxBeginSelect %W [%W index @%x,%y]
+        %W activate @%x,%y
+        %W selection clear 0 end
+        %W selection set active
         tk_popup .menu.sel.m %X %Y
     }
 
-#    set_focus_style $p.query.list
-#    if {$TkGnats(edit_authorized)} {
-#        bind  $p.query.list e "selection_Edit_cmd $p.query.list"
-#    }
-#    bind  $p.query.list v "selection_View_Formatted_cmd %W"
+    set_focus_style $p.query.list
+    if {$TkGnats(edit_authorized)} {
+        bind  $p.query.list e "selection_Edit_cmd   $p.query.list"
+        bind  $p.query.list E "selection_Edit_cmd   $p.query.list"
+    }
+    if {$TkGnats(delete_authorized)} {
+        bind  $p.query.list d "selection_Delete_cmd $p.query.list"
+        bind  $p.query.list D "selection_Delete_cmd $p.query.list"
+    }
+    bind  $p.query.list v "selection_View_Formatted_cmd %W"
+    bind  $p.query.list V "selection_View_Formatted_cmd %W"
+    bind  $p.query.list r "selection_View_Raw_cmd %W"
+    bind  $p.query.list R "selection_View_Raw_cmd %W"
+    bind  $p.query.list h "selection_Remove_cmd %W"
+    bind  $p.query.list H "selection_Remove_cmd %W"
+    bind  $p.query.list m "selection_Email_cmd %W"
+    bind  $p.query.list M "selection_Email_cmd %W"
 
     return $p.query.list
 }
 
+proc query_listbox_xscroll {args} {
+    global Query list_flds_heading
+    eval [lindex $args 0] set [lrange $args 2 end]
+    set  first [expr int([lindex [[lindex $args 1] xview] 0] * $Query(maxlen) + .5)]
+    set Query(lbheading) [string range $list_flds_heading $first end]
+}
+
 #
 # ---- Callbacks
 #
@@ -702,7 +764,7 @@ proc selection_Delete_cmd {w} {
     set ln [selln $w]
     if {"$ln" != ""} {
 	set prid [pridfromsummaryline $ln]
-        . configure -cursor watch
+        busy_cursor set
 	headingMsg "Please Wait..."
         set result [delete_pr $prid]
         if {[string first deleted $result] >= 0} {
@@ -716,7 +778,7 @@ proc selection_Delete_cmd {w} {
             }
             headingMsg "Delete failed."
         }
-        . configure -cursor left_ptr
+        busy_cursor clear
     }
     if {[$w size] == 0} {
         disable_listbox_menus
@@ -726,7 +788,7 @@ proc selection_Delete_cmd {w} {
 proc selection_Email_cmd {w} {
     set ln [selln $w]
     if {"$ln" != ""} {
-        . configure -cursor watch
+        busy_cursor set
 	headingMsg "Please Wait..."
 	set prid [pridfromsummaryline $ln]
         if {[set prtext  [get_pr_full_text $prid]] == "-1"} {
@@ -734,10 +796,10 @@ proc selection_Email_cmd {w} {
         } {
             set time 2250
             parsepr_txt $prtext flds
-            email_originator [ftrim $flds(Reply-To)] [ftrim $flds(>Responsible)] \
-                    [ftrim $flds(From)] [ftrim $flds(>Category)]/$prid [ftrim $flds(>Synopsis)]
+            email_originator [ftrim $flds(X-GNATS-Notify)] [ftrim $flds(>Responsible)] \
+                    [ftrim $flds(Reply-To)] [ftrim $flds(>Category)]/$prid [ftrim $flds(>Synopsis)]
         }
-	after $time {headingMsg " " ; . configure -cursor left_ptr}
+	after $time {headingMsg " " ; busy_cursor clear}
     }
 }
 
@@ -746,7 +808,7 @@ proc selection_Edit_cmd {w} {
     set ln [selln $w]
     if {"$ln" != ""} {
 	headingMsg "Please Wait..."
-        . configure -cursor watch
+        busy_cursor set
 	set prid [pridfromsummaryline $ln]
 	TkGnats_exec $TkGnats(WISHPATH) $TkGnats(lib)/tkeditpr.tcl -prid $prid \
                 -server $TkGnats(ServerInfo) \
@@ -754,7 +816,7 @@ proc selection_Edit_cmd {w} {
                 -categories $TkGnats(CategoryList) -submitters $TkGnats(SubmitterList) \
                 -responsible $TkGnats(ResponsibleFile) &
 	schedule_reap
-	after 2250 {headingMsg " " ; . configure -cursor left_ptr}
+	after 2250 {headingMsg " " ; busy_cursor clear}
     }
 }
 
@@ -763,7 +825,7 @@ proc selection_View_Formatted_cmd {w} {
     set ln [selln $w]
     if {"$ln" != ""} {
 	headingMsg "Please Wait..."
-        . configure -cursor watch
+        busy_cursor set
 	set prid [pridfromsummaryline $ln]
 	TkGnats_exec $TkGnats(WISHPATH) $TkGnats(lib)/tkviewpr.tcl -prid $prid \
                 -server $TkGnats(ServerInfo) \
@@ -771,7 +833,7 @@ proc selection_View_Formatted_cmd {w} {
                 -categories $TkGnats(CategoryList) -submitters $TkGnats(SubmitterList) \
                 -responsible $TkGnats(ResponsibleFile) &
 	schedule_reap
-	after 2250 {headingMsg " " ; . configure -cursor left_ptr}
+	after 2250 {headingMsg " " ; busy_cursor clear}
     }
 }
 
@@ -779,7 +841,7 @@ proc selection_View_Raw_cmd {w} {
     global TkGnats
     set ln [selln $w]
     if {"$ln" != ""} {
-        . configure -cursor watch
+        busy_cursor set
 	headingMsg "Please Wait..."
 	set prid [pridfromsummaryline $ln]
 	TkGnats_exec $TkGnats(WISHPATH) $TkGnats(lib)/tkviewpr.tcl -prid $prid -raw 1 \
@@ -788,7 +850,7 @@ proc selection_View_Raw_cmd {w} {
                 -categories $TkGnats(CategoryList) -submitters $TkGnats(SubmitterList) \
                 -responsible $TkGnats(ResponsibleFile) &
 	schedule_reap
-	after 2250 {headingMsg " " ; . configure -cursor left_ptr}
+	after 2250 {headingMsg " " ; busy_cursor clear}
     }
 }
 
@@ -928,13 +990,6 @@ proc build_dumb_query_qualifier {subclauseop type tag lst} {
     append Query(default__query) ") && \\"
 }
 
-# convert from query-pr "yy-mm-dd hh:mm" format to "mm/dd/yy hh:mm" for Tcl clock command
-proc convert_date_format {date} {
-    set dt  [split $date]
-    set d   [split [lindex $dt 0] -]
-    return "[lindex $d 1]/[lindex $d 2]/[lindex $d 0] [lindex $dt 1]"
-}
-
 proc get_last_modified {arr_date last_mod} {
     if {$last_mod != ""} {
         set date $last_mod
@@ -964,33 +1019,55 @@ proc XXXbuild_mtime_query_qualifier {mtime} {
 }
 
 proc save_query_listbox { sfout s } {
-    case $s Category {
-	set ew .eboxs.clb
-    } Submitter {
-	set ew .eboxs.slb
-    } Responsible {
-	set ew .eboxs.rlb
+    switch $s {
+        Category {
+            set ew .eboxs.clb
+        }
+        Submitter-Id {
+            set ew .eboxs.slb
+        }
+        Responsible {
+            set ew .eboxs.rlb
+        }
     }
+
+    if {![winfo exists $ew]} {
+        return
+    }
+    
     puts $sfout "load_query_listbox $s \{[textget RegExp $ew.reg]\}"
 }
 
 proc load_query_listbox { s {re {}}} {
     global TkGnats $s
-    case $s Category {
-	set lb .eboxs.clb.cat
-	set ew .eboxs.clb
-    } Submitter {
-	set lb .eboxs.slb.sub
-	set ew .eboxs.slb
-    } Responsible {
-	set lb .eboxs.rlb.res
-	set ew .eboxs.rlb
+    switch $s {
+        Category {
+            set lb .eboxs.clb.cat
+            set ew .eboxs.clb
+            set list Category
+        }
+        # Submitter is for compatibility with old saved queries prior to tkgnats-3.0.12
+        Submitter -
+        Submitter-Id {
+            set lb .eboxs.slb.sub
+            set ew .eboxs.slb
+            set list Submitter
+        }
+        Responsible {
+            set lb .eboxs.rlb.res
+            set ew .eboxs.rlb
+            set list Responsible
+        }
+    }
+
+    if {![winfo exists $lb]} {
+        return
     }
     
     $lb.l.list delete 0 end
     $lb.r.list delete 0 end
 
-    foreach a $TkGnats(${s}List) {
+    foreach a $TkGnats(${list}List) {
 	if {[lsearch [array names $s] $a] < 0} {
 	    $lb.l.list insert end $a
 	} {
@@ -1005,6 +1082,9 @@ proc load_query_listbox { s {re {}}} {
 
 proc load_query_entry { s re {when {}}} {
     global Query
+    if {![info exists Query(stextparent,$s)]} {
+        return
+    }
     if {$when == ""} {
         textset $s $re $Query(stextparent,$s)
     } {
@@ -1013,8 +1093,7 @@ proc load_query_entry { s re {when {}}} {
 }
 
 proc get_save_file_name {title} {
-    set origp [entryDialog "Enter name of file to save $title into:\n\n(Hint: underscores in name are replaced\nwith blanks for nice looking menu names)"]
-    if {"$origp" == ""} {
+    if {[catch {entryDialog "Enter name of file to save $title into:\n\n(Hint: underscores in name are replaced\nwith blanks for nice looking menu names)" Cancel "" 0 .} origp]} {
         return ""
     }
     set p [string trim $origp " \t\n!;'<>?*%$#"]
@@ -1081,9 +1160,10 @@ proc save_query_cmd {mode} {
 
     # for array globals
     
-    foreach f {
-	State Priority Confidential Category Severity Class Responsible Submitter
-    } {
+    foreach f {State Priority Confidential Category Severity Class Responsible Submitter-Id} {
+        if {[check_suppressed_field $f]} {
+            continue
+        }
 	global $f
         if {$mode == "save"} {
             foreach a [array names $f] {
@@ -1109,13 +1189,16 @@ proc save_query_cmd {mode} {
     
     if {$mode == "save"} {
         save_query_listbox $sfout Category
-        save_query_listbox $sfout Submitter
+        save_query_listbox $sfout Submitter-Id
         save_query_listbox $sfout Responsible
     }
 
     # the text field regexp values
     
-    foreach f "Synopsis Days-idle Originator Text-Fields Release Keywords $TkGnats(Quarter)" {
+    foreach f "Synopsis Days-Idle Originator Text-Fields Release Keywords $TkGnats(Quarter)" {
+        if {[check_suppressed_field $f]} {
+            continue
+        }
         if {$mode == "save"} {
             if {[info exists Query(stextparent,$f)] && "[textget $f $Query(stextparent,$f)]" != ""} {
                 puts $sfout "load_query_entry $f \{[textget $f $Query(stextparent,$f)]\}"
@@ -1144,7 +1227,7 @@ proc save_query_cmd {mode} {
                         build_regex_query_qualifier -or -glob $tag {} \
                                 [textget $f $Query(stextparent,$f)]
                     }
-                    Days-idle {
+                    Days-Idle {
                         build_mtime_query_qualifier [textget $f $Query(stextparent,$f)]
                     }
                     default {
@@ -1157,9 +1240,10 @@ proc save_query_cmd {mode} {
 
     # the date field values
     
-    foreach f {
-	Arrival-Date Last-Modified Closed-Date Date-Required
-    } {
+    foreach f {Arrival-Date Last-Modified Closed-Date Date-Required} {
+        if {[check_suppressed_field $f]} {
+            continue
+        }
         if {$mode == "save"} {
             if {[info exists Query(stextparent,$f)] && "[textget $f $Query(stextparent,$f) after]" != ""} {
                 puts $sfout "load_query_entry $f \{[textget $f $Query(stextparent,$f) after]\} after"
@@ -1194,9 +1278,7 @@ proc clear_query_cmd {{headingmsgflag ""}} {
     }
 
     # for array globals
-    foreach f {
-	State Priority Confidential Category Severity Class Responsible Submitter
-    } {
+    foreach f {State Priority Confidential Category Severity Class Responsible Submitter-Id} {
 	global $f
         foreach a [array names $f] {
             set [set f]($a) {}
@@ -1204,35 +1286,19 @@ proc clear_query_cmd {{headingmsgflag ""}} {
     }
 
     # Query listboxes
-    category_listbox    .eboxs.clb
-    submitter_listbox   .eboxs.slb
-    responsible_listbox .eboxs.rlb
-    textset RegExp ""   .eboxs.clb.reg
-    textset RegExp ""   .eboxs.slb.reg
-    textset RegExp ""   .eboxs.rlb.reg
-
-    # the text field values
-    foreach f {
-	Synopsis Days-idle Originator Text-Fields Release Number
-    } {
-        textset $f "" $Query(stextparent,$f)
+    foreach {efield eframe} {Category clb Submitter-Id slb Responsible rlb} {
+        if {![check_suppressed_field $efield]} {
+            [string tolower $efield]_listbox .eboxs.$eframe
+            textset RegExp ""    .eboxs.$eframe.reg
+        }
     }
 
-    # the date fields
-    foreach f {
-        Arrival-Date Last-Modified Closed-Date
-    } {
-        textset $f "" $Query(stextparent,$f) after
-        textset $f "" $Query(stextparent,$f) before
+    foreach {f t} [list Synopsis text Days-Idle text Originator text Text-Fields text Release text Number text Arrival-Date after Arrival-Date before Last-Modified after Last-Modified before Closed-Date after Closed-Date before Keywords text $TkGnats(Quarter) text Date-Required after Date-Required before] {
+        if {[info  exists  Query(stextparent,$f)]} {
+            textset $f "" $Query(stextparent,$f) $t
+        }
     }
-    
-    if {$TkGnats(ReleaseBased)} {
-	textset Keywords          "" $Query(stextparent,Keywords)
-	textset $TkGnats(Quarter) "" $Query(stextparent,$TkGnats(Quarter))
-	textset Date-Required     "" $Query(stextparent,Date-Required) after
-	textset Date-Required     "" $Query(stextparent,Date-Required) before
-    }
-
+        
     # the query listbox
     set prid [pridfromsummaryline [selln $lbpath]]
     if {$prid > 0} {
@@ -1267,6 +1333,7 @@ proc query_fill_listbox {} {
     }
     $lbpath delete 0 end; # clear current list
     set c 0
+    set Query(maxlen) 0
     foreach ln $Query(PrList) {
 	incr c
 	if {"$ln" == ""} {
@@ -1383,6 +1450,10 @@ proc query_fill_listbox {} {
                     }
                 }
 		$lbpath insert end $ln
+                set maxlen [string length $ln]
+                if {$Query(maxlen)  < $maxlen} {
+                    set Query(maxlen) $maxlen
+                }
 	    }
 	}
     }
@@ -1578,21 +1649,19 @@ proc perform_query_cmd {{query _list_} {prid {}}} {
     if {[TkGnats_UpdateCheck]} {
 	return
     }
-    . configure -cursor watch
-    set entry_cursor [[lindex $Query(tlist) 0] cget -cursor]
-    foreach w $Query(tlist) {
-        $w configure -cursor watch
-    }
-
+    busy_cursor set
     if {[string compare $Query(done_msg) ""] == 0} {
         set Query(done_msg) Query
     }
     headingMsg "Performing $Query(done_msg)..."
 
-    case $query {_list_} {
-	query_cmd
-    } prid {
-	query_cmd $prid
+    switch $query {
+        _list_ {
+            query_cmd
+        }
+        prid {
+            query_cmd $prid
+        }
     }
 
     sort_cmd
@@ -1600,10 +1669,7 @@ proc perform_query_cmd {{query _list_} {prid {}}} {
     
     headingMsg "Done $Query(done_msg):   [$lbpath size] matches found"
     set Query(done_msg) ""
-    . configure -cursor left_ptr
-    foreach w $Query(tlist) {
-        $w configure -cursor $entry_cursor
-    }
+    busy_cursor clear
     return
 }
 
@@ -1623,30 +1689,44 @@ proc perform_print_cmd {{print _list_}} {
         return
     }
     
-    . configure -cursor watch
+    busy_cursor set
 
     if {![info exists TkGnats(MsMacroSet)]} {
 	set TkGnats(MsMacroSet) "-ms"
     }
     
-    case $Print(Device) previewer {
-	set previewfile /tmp/tkq.ps.$TkGnats(LogName)
-        if {[string compare $Print(Format) "troff"] == 0} {
-            set fout [open $previewfile w]
-        } {
-            set fout [open "|groff -t -T$Print(Format) $TkGnats(MsMacroSet) > $previewfile" w]
+    switch $Print(Device) {
+        previewer {
+            set previewfile /tmp/tkq.ps.$TkGnats(LogName)
+            if {[string compare $Print(Format) "troff"] == 0} {
+                set fout [open $previewfile w]
+            } {
+                if {[string compare $Print(Format) "ascii"] == 0 || \
+                        [string compare $Print(Format) "latin1"] == 0} {
+                    set fout [open "|groff -t -T$Print(Format) $TkGnats(MsMacroSet) -P-b > $previewfile" w]
+                } {
+                    set fout [open "|groff -t -T$Print(Format) $TkGnats(MsMacroSet)      > $previewfile" w]
+                }
+            }
         }
-    } printer {
-        if {[string compare $Print(Format) "troff"] == 0} {
-            set fout [open "| $Print(PrintSpooler,$Print(Format))" w]
-        } {
-            set fout [open "|groff -t -T$Print(Format) $TkGnats(MsMacroSet) | $Print(PrintSpooler,$Print(Format))" "w"]
+        printer {
+            if {[string compare $Print(Format) "troff"] == 0} {
+                set fout [open "| $Print(PrintSpooler,$Print(Format))" w]
+            } {
+                set fout [open "|groff -t -T$Print(Format) $TkGnats(MsMacroSet) | $Print(PrintSpooler,$Print(Format))" "w"]
+            }
         }
-    } file {
-        if {[string compare $Print(Format) "troff"] == 0} {
-            set fout [open $Print(savefile) w]
-        } {
-            set fout [open "|groff -t -T$Print(Format) $TkGnats(MsMacroSet) > $Print(savefile)" w]
+        file {
+            if {[string compare $Print(Format) "troff"] == 0} {
+                set fout [open $Print(savefile) w]
+            } {
+                if {[string compare $Print(Format) "ascii"] == 0 || \
+                        [string compare $Print(Format) "latin1"] == 0} {
+                    set fout [open "|groff -t -T$Print(Format) $TkGnats(MsMacroSet) -P-b > $Print(savefile)" w]
+                } {
+                    set fout [open "|groff -t -T$Print(Format) $TkGnats(MsMacroSet)      > $Print(savefile)" w]
+                }
+            }
         }
     }
 
@@ -1654,7 +1734,7 @@ proc perform_print_cmd {{print _list_}} {
 
     # If TkGnats(GNATS_ACCESS_METHOD) != "socket" then this does nothing
     if {[open_socket_gnatsd 1] == "-1"} {
-        . configure -cursor left_ptr
+        busy_cursor clear
         return
     }
     
@@ -1672,14 +1752,14 @@ proc perform_print_cmd {{print _list_}} {
     
     headingMsg "$done_msg"
     set Query(done_msg) ""
-    . configure -cursor left_ptr
+    busy_cursor clear
     return
 }
 
 proc dump_listbox_data {} {
     global Query
     headingMsg "Saving Listbox Dump..."
-    . configure -cursor watch
+    busy_cursor set
     set filename  [tk_getSaveFile]
     if {$filename == ""} {
         headingMsg "Saving Listbox Dump...Cancelled"
@@ -1687,7 +1767,7 @@ proc dump_listbox_data {} {
         file_put_text $filename [join $Query(PrList) \n]
         headingMsg "Saving Listbox Dump...done (written to $filename)"
     }
-    . configure -cursor left_ptr
+    busy_cursor clear
 }
 
 proc set_query_sorting_cmd {} {
@@ -1755,6 +1835,9 @@ menu       .mframe.file.m
 
 menubutton .mframe.edit -text "Edit" -menu .mframe.edit.m -underline 0
 menu       .mframe.edit.m
+.mframe.edit.m     configure -disabledforeground [.mframe.edit.m cget -foreground]
+.mframe.edit.m add command -label "Use right mouse button for Cut/Copy/Paste" -state disabled
+.mframe.edit.m add separator
 .mframe.edit.m add command -label "Fonts..." -command "edit_fonts"
 
 pack .mframe.file .mframe.edit -side left
@@ -1764,6 +1847,9 @@ menu       .mframe.help.m
 .mframe.help.m add command -label "Overview" \
         -command "helpMsg Query_Overview"
 .mframe.help.m add separator
+.mframe.help.m add command -label "Cut, Copy, Paste Operations" \
+        -command "helpMsg Cut_Copy_Paste"
+.mframe.help.m add separator
 .mframe.help.m add command -label "Field Definitions" \
         -command "helpMsg Field_Definitions"
 .mframe.help.m add command -label "Regular Expressions" \
@@ -1783,88 +1869,94 @@ menu       .mframe.help.m
 .mframe.help.m add command -label "View Configuration Variables" \
         -command "helpMsg TkGnats_Variables"
 .mframe.help.m add separator
+.mframe.help.m add command -label "Changes" \
+        -command "helpMsg Changes"
 .mframe.help.m add command -label "About" \
         -command "helpMsg TkGnats_About"
 pack .mframe.help -side right
 
 frame .eflds
 radiobar_frame .eflds .eflds.lb
-#######checkbar .eflds.lb class Class {sw-bug doc-bug change-request support mistaken duplicate} All
-checkbar .eflds.lb class Class $Mappings(Class) All
-#######checkbar .eflds.lb state State {open analyzed feedback closed suspended} All
-checkbar .eflds.lb state State $Mappings(State) All
-checkbar .eflds.lb priority Priority {low medium high} All
-checkbar .eflds.lb severity Severity {non-critical serious critical} All
-checkbar .eflds.lb confidential Confidential {yes no} All
+foreach {efield elist} [list Class $TkGnats(ClassesList) State $TkGnats(StatesList) Priority {low medium high} Severity {non-critical serious critical} Confidential {no yes}] {
+    if {![check_suppressed_field $efield]} {
+        checkbar .eflds.lb [string tolower $efield] $efield $elist All
+    }
+}
+pack .eflds -side top  -padx  0 -fill x -anchor w
 
-frame .eboxs
-frame .eboxs.clb -relief groove -borderwidth 2
-set   cbpath [category_listbox    .eboxs.clb]
-frame .eboxs.slb -relief groove -borderwidth 2
-set   sbpath [submitter_listbox   .eboxs.slb]
-frame .eboxs.rlb -relief groove -borderwidth 2
-set   rbpath [responsible_listbox .eboxs.rlb]
+frame .eboxs    
+set   nboxes 1
+array set xpad {1 0 2 20 3 0}
+foreach {efield eframe epath} {Category clb cbpath Submitter-Id slb sbpath Responsible rlb rbpath} {
+    if {![check_suppressed_field $efield]} {
+        frame .eboxs.$eframe -relief groove -borderwidth 2
+        set   $epath [[string tolower $efield]_listbox .eboxs.$eframe]
+        pack .eboxs.$eframe -side left -padx $xpad($nboxes)
+        incr nboxes
+    }
+}
+if {$nboxes > 1} {
+    pack .eboxs     -side top  -pady  4 -fill x -anchor w
+}
 
-pack .eboxs.clb -side left -padx  0
-pack .eboxs.slb -side left -padx 20
-pack .eboxs.rlb -side left -padx  0
-pack .eflds     -side top  -padx  0 -fill x -anchor w
-pack .eboxs     -side top  -pady  4 -fill x -anchor w
+set tlist [list Number Release Synopsis Originator Text-Fields Days-Idle]
+if {$TkGnats(ReleaseBased)} {
+    lappend tlist Keywords $TkGnats(Quarter)
+}
+set textlist ""
+set ntlist   0
+foreach f $tlist {
+    if {![check_suppressed_field $f]} {
+        lappend textlist $f
+        incr ntlist
+    }
+}
+
+set dlist [list Arrival-Date Last-Modified Closed-Date]
+if {$TkGnats(ReleaseBased)} {
+    lappend dlist Date-Required
+}
+set datelist ""
+set ndlist   0
+foreach f $dlist {
+    if {![check_suppressed_field $f]} {
+        lappend datelist $f
+        incr ndlist
+    }
+}
 
 frame .stext   -relief flat
 frame .stext.l -relief flat
-frame .stext.c -relief flat
-frame .stext.r -relief flat
-pack  .stext            -side top  -fill x    -expand no  -anchor w
-pack  .stext.l .stext.c -side left -fill x    -expand yes -anchor nw
-pack  .stext.r          -side left -fill none -expand no  -anchor nw
+pack  .stext   -side top  -fill x    -expand no  -anchor w
+pack  .stext.l -side left -fill x    -expand yes -anchor nw
+if {$ntlist <= $ndlist} {
+    set frames  .stext.l
+    set nframes 1
+} {
+    set frames  [list .stext.l .stext.c]
+    set nframes 2
+    frame .stext.c -relief flat
+    pack  .stext.c -side left -fill x    -expand yes -anchor nw
+}
+if {$ndlist > 0} {
+    frame .stext.r -relief flat
+    pack  .stext.r     -side left -fill none -expand no  -anchor nw
+}
+
+set n 0
+foreach f $textlist {
+    set Query(stextparent,$f) [lindex $frames [expr $n % $nframes]]
+    lappend Query(tlist) [singletext $Query(stextparent,$f) $f 20 "" 12]
+    incr n
+}
+foreach f $datelist {
+    set Query(stextparent,$f) .stext.r
+    eval lappend Query(tlist) [daterange $Query(stextparent,$f) $f 16 "" "" 14]
+}
 
 set Query(curr_prid) 0
 
-set Query(stextparent,Number)        .stext.l
-set Query(stextparent,Synopsis)      .stext.l
-set Query(stextparent,Text-Fields)   .stext.l
-set Query(stextparent,Release)       .stext.c
-set Query(stextparent,Originator)    .stext.c
-set Query(stextparent,Days-idle)     .stext.c
-set Query(stextparent,Arrival-Date)  .stext.r
-set Query(stextparent,Last-Modified) .stext.r
-set Query(stextparent,Closed-Date)   .stext.r
-
-#TTD remove me after testing
-#set TkGnats(ReleaseBased) 1
-
-if {$TkGnats(ReleaseBased)} {
-    set Query(stextparent,Keywords)          .stext.l
-    set Query(stextparent,$TkGnats(Quarter)) .stext.c
-    set Query(stextparent,Date-Required)     .stext.r
-}
-
-lappend Query(tlist) [singletext $Query(stextparent,Number)      Number      20 "" 12]
-lappend Query(tlist) [singletext $Query(stextparent,Release)     Release     20 "" 11]
-set     Query(tlist) [concat $Query(tlist) \
-        [daterange   $Query(stextparent,Arrival-Date)        Arrival-Date    16 "" "" 14]]
-
-lappend Query(tlist) [singletext $Query(stextparent,Synopsis)    Synopsis    20 "" 12]
-lappend Query(tlist) [singletext $Query(stextparent,Originator)  Originator  20 "" 11]
-set     Query(tlist) [concat $Query(tlist) \
-        [daterange   $Query(stextparent,Last-Modified)       Last-Modified   16 "" "" 14]]
-
-lappend Query(tlist) [singletext $Query(stextparent,Text-Fields) Text-Fields 20 "" 12]
-lappend Query(tlist) [singletext $Query(stextparent,Days-idle)   Days-idle   20 "" 11]
-set     Query(tlist) [concat $Query(tlist) \
-        [daterange   $Query(stextparent,Closed-Date)         Closed-Date     16 "" "" 14]]
-
-
-if {$TkGnats(ReleaseBased)} {
-    lappend Query(tlist) [singletext $Query(stextparent,Keywords) Keywords   20 "" 12]
-    lappend Query(tlist) [singletext $Query(stextparent,$TkGnats(Quarter)) \
-            $TkGnats(Quarter) 20 "" 11]
-    set     Query(tlist) [concat $Query(tlist) \
-            [daterange   $Query(stextparent,Date-Required)   Date-Required   16 "" "" 14]]
-}
-    
-set_text_traversal $Query(tlist)
+# Do this before adding the listbox to Query(tlist); we don't want Return bound there.
 foreach w $Query(tlist) {
     bind $w <Return> "perform_query_cmd"
 }
@@ -1879,6 +1971,9 @@ pack  .menu -side top -fill x -pady 4
 frame .qlb
 pack  .qlb -expand true -fill both
 set   lbpath [query_listbox .qlb]
+lappend Query(tlist) $lbpath
+
+set_text_traversal $Query(tlist)
 
 #
 # Query Menu
@@ -2011,8 +2106,15 @@ menu .menu.view.m.save -tearoff no
 set list_flds_selected(Number) Number
 for {set i 2} {$i < [llength $list_flds_list]} {incr i 2} {
     set x [lindex $list_flds_list $i]
-    .menu.view.m add checkbutton -label $x -command "update_query_view_fields" \
-            -offvalue "" -onvalue $x -variable [format "list_flds_selected(%s)" $x]
+    #if {![check_suppressed_field $x]} {
+    #    .menu.view.m add checkbutton -label $x -command "update_query_view_fields" \
+    #            -offvalue "" -onvalue $x -variable [format "list_flds_selected(%s)" $x]
+    #}
+    if {![check_suppressed_field $x]} {
+        .menu.view.m add checkbutton -label [get_field_alias $x] \
+                -command "update_query_view_fields" \
+                -offvalue "" -onvalue $x -variable [format "list_flds_selected(%s)" $x]
+    }
 }
 
 #
@@ -2074,25 +2176,25 @@ foreach x [get_merge_save_dirs $TkGnats(lib)/print $Tkprfolder(site_print_dir)]
 menubutton .menu.sel -text "Actions" -menu .menu.sel.m -underline 0
 menu       .menu.sel.m
 
-.menu.sel.m add command -label "Edit..."           -command "selection_Edit_cmd $lbpath"
+.menu.sel.m add command -label "Edit..."           -command "selection_Edit_cmd $lbpath" -accel e
 if {! $TkGnats(edit_authorized)} {
     .menu.sel.m entryconfigure "Edit..." -state disabled
 }
-.menu.sel.m add command -label "View..."           -command "selection_View_Formatted_cmd $lbpath"
-.menu.sel.m add command -label "View Raw Data..."  -command "selection_View_Raw_cmd       $lbpath"
+.menu.sel.m add command -label "View..."           -command "selection_View_Formatted_cmd $lbpath" -accel v
+.menu.sel.m add command -label "View Raw Data..."  -command "selection_View_Raw_cmd       $lbpath" -accel r
 
 .menu.sel.m add separator
 
 if {$TkGnats(delete_authorized)} {
-    .menu.sel.m add command -label "Delete PERMANENTLY" -command "selection_Delete_cmd $lbpath"
+    .menu.sel.m add command -label "Delete PERMANENTLY" -command "selection_Delete_cmd $lbpath" -accel d
     .menu.sel.m add separator
 }
 
-.menu.sel.m add command -label "Remove From List"  -command "selection_Remove_cmd $lbpath"
+.menu.sel.m add command -label "Hide From List"    -command "selection_Remove_cmd $lbpath" -accel h
 
 .menu.sel.m add separator
 
-.menu.sel.m add command -label "Send Email..."     -command "selection_Email_cmd  $lbpath"
+.menu.sel.m add command -label "Send Email..."     -command "selection_Email_cmd  $lbpath" -accel m
 
 button .menu.doquery -text "Do Query" -command perform_query_cmd
 button .menu.clear   -text "Clear"    -command clear_query_cmd
